# -*- coding: utf-8 -*-

import unittest

import ldap3

import pytest

import lgmm

from .common import (
    MockLIGOLDAPWithManager,
    logger,
)
from .data import (
    data,
    directory,
)


@pytest.mark.skipif(
    ldap3.__version__ < "2.5.0",
    reason="ldap3 too old (< 2.5.0)",
)
class CentralManagerLIGOTestsSources(MockLIGOLDAPWithManager):
    """
    Test CentralManager functionality using standard LIGO schema but with different sources
    """

    directory = directory

    @unittest.skip("Can only check for one LDAP server")
    def test_multiple_ldap_urls(self):
        manager = lgmm.MapfileManager(logger=logger)
        manager.LDAP_URLS = ["ldaps://0.0.0.0/", "ldaps://ldap.ligo.org"]
        mapping = lgmm.CentralMapping(self.manager)

        self.assertEqual(3, len(mapping))

        self.assertIn(data["einstein_dn"], mapping)

    def test_ldap_base_and_scope(self):
        self.manager.SOURCES = ["people", "robots"]

        self.manager.LDAP_SCOPE = "onelevel"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(0, len(mapping))

        self.manager.LDAP_BASE_people = "ou=people,dc=ligo,dc=org"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(3, len(mapping))
        self.assertIn(data["einstein_dn"], mapping)

        self.manager.LDAP_BASE_robots = "ou=robots,dc=ligo,dc=org"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(3, len(mapping))

    def test_groups(self):
        self.manager.SOURCES = ["people", "robots"]

        self.manager.GROUPS_robots = [["Communities:Robots"]]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(4, len(mapping))
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

    def test_altsyntax(self):
        self.manager.SOURCES = ["people", "altpeople"]
        self.manager.LDAP_BASE_altpeople = "dc=ligo-alt,dc=org"
        self.manager.MEMBER_ATTR_altpeople = "groupMembership"
        self.manager.LOGIN_ATTR_altpeople = "uid"
        self.manager.SUBJECT_ATTR_altpeople = "sn"
        self.manager.SUBJECT_TRANSFORM_altpeople = "NONE"

        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(6, len(mapping))
        self.assertEqual(mapping.get(data["einstein_dn"]), data["einstein_usernames"])
        self.assertEqual(mapping.get("Einstein"), data["einstein_usernames"])

    def test_altsyntax_with_site(self):
        self.manager.SOURCES = ["people", "altpeople"]
        self.manager.SITE_people = "cdf"

        self.manager.LDAP_BASE_altpeople = "dc=ligo-alt,dc=org"
        self.manager.MEMBER_ATTR_altpeople = "groupMembership"
        self.manager.LOGIN_ATTR_altpeople = "uid"
        self.manager.SUBJECT_ATTR_altpeople = "sn"
        self.manager.SUBJECT_TRANSFORM_altpeople = "NONE"
        self.manager.SITE_altpeople = "cardiff"
        self.manager.SITE_ATTR_altpeople = "x-private-{site}-attr"

        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(6, len(mapping))
        self.assertEqual(
            mapping.get(data["einstein_dn"]), data["einstein_usernames_cdf"]
        )
        self.assertEqual(mapping.get("Einstein"), data["einstein_usernames_cdf"])

    def test_username_policy(self):
        self.manager.SOURCES = ["people", "altpeople"]
        self.manager.SITE = "cdf"

        self.manager.USERNAME_POLICY_altpeople = "SITE_FIRST"
        # Required for altsyntax
        self.manager.LDAP_BASE_altpeople = "dc=ligo-alt,dc=org"
        self.manager.MEMBER_ATTR_altpeople = "groupMembership"
        self.manager.LOGIN_ATTR_altpeople = "uid"
        self.manager.SUBJECT_ATTR_altpeople = "sn"
        self.manager.SUBJECT_TRANSFORM_altpeople = "NONE"

        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(6, len(mapping))
        self.assertEqual(
            mapping.get(data["einstein_dn"]), data["einstein_usernames_cdf"]
        )
        self.assertEqual(mapping.get("Einstein"), data["einstein_usernames"])

    def test_login_whitelist(self):
        # Get all people and robots individually
        self.manager.SOURCES = ["people", "robots"]
        self.manager.LDAP_SCOPE = "onelevel"
        self.manager.LDAP_BASE_people = "ou=people,dc=ligo,dc=org"
        self.manager.LDAP_BASE_robots = "ou=robots,dc=ligo,dc=org"
        self.manager.GROUPS = [[]]

        self.manager.LOGIN_WHITELIST_people = [""]
        self.manager.LOGIN_WHITELIST_robots = []
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(mapping.get(data["einstein_dn"]), [])
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

        self.manager.LOGIN_WHITELIST_people = [""]
        self.manager.LOGIN_WHITELIST_robots = ["robot"]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(mapping.get(data["einstein_dn"]), [])
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

        self.manager.LOGIN_WHITELIST_people = ["albert"]
        self.manager.LOGIN_WHITELIST_robots = ["robot"]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(mapping.get(data["einstein_dn"]), ["albert"])
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

    def test_filter(self):
        # Get all people and robots individually
        self.manager.SOURCES = ["people", "robots"]
        self.manager.LDAP_SCOPE = "onelevel"
        self.manager.LDAP_BASE_people = "ou=people,dc=ligo,dc=org"
        self.manager.LDAP_BASE_robots = "ou=robots,dc=ligo,dc=org"
        self.manager.GROUPS = [[]]

        self.manager.FILTER = "uid=nobody"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(0, len(mapping))

        self.manager.FILTER_people = "uid=albert.einstein"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(1, len(mapping))
        self.assertEqual(mapping.get(data["einstein_dn"]), data["einstein_usernames"])

        self.manager.FILTER_robots = "uid=robot"
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(2, len(mapping))
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

    def test_overrides(self):
        # Get all people and robots individually
        self.manager.SOURCES = ["people", "robots"]
        self.manager.LDAP_SCOPE = "onelevel"
        self.manager.LDAP_BASE_people = "ou=people,dc=ligo,dc=org"
        self.manager.LDAP_BASE_robots = "ou=robots,dc=ligo,dc=org"
        self.manager.GROUPS = [[]]

        self.manager.OVERRIDES = []
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(mapping.get(data["einstein_dn"]), data["einstein_usernames"])
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

        self.manager.OVERRIDES_people = [{"conditions": {}, "logins_append": ["ligo"]}]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(
            mapping.get(data["einstein_dn"]), data["einstein_usernames"] + ["ligo"]
        )
        self.assertEqual(mapping.get(data["robot_dn"]), data["robot_usernames"])

        self.manager.OVERRIDES_robots = [{"conditions": {}, "logins_append": ["ligo"]}]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(
            mapping.get(data["einstein_dn"]), data["einstein_usernames"] + ["ligo"]
        )
        self.assertEqual(
            mapping.get(data["robot_dn"]), data["robot_usernames"] + ["ligo"]
        )

    def test_login_groups_map(self):
        # Get all people and robots individually
        self.manager.SOURCES = ["people", "robots"]
        self.manager.LDAP_SCOPE = "onelevel"
        self.manager.LDAP_BASE_people = "ou=people,dc=ligo,dc=org"
        self.manager.LDAP_BASE_robots = "ou=robots,dc=ligo,dc=org"
        self.manager.GROUPS = [[]]

        self.manager.LOGIN_GROUPS_MAP = []
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(mapping.get(data["einstein_dn"]), data["einstein_usernames"])

        self.manager.LOGIN_GROUPS_MAP_people = [
            (["cbc"], ["Communities:LVC:LSC:LDG:Accounts:CBC:LDGAccountsCBCCBCUsers"])
        ]
        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(
            mapping.get(data["einstein_dn"]), data["einstein_usernames"] + ["cbc"]
        )

        self.manager.LOGIN_GROUPS_MAP_robots = [(["cbc"], ["Communities:Robots"])]

        mapping = lgmm.CentralMapping(self.manager)
        self.assertEqual(5, len(mapping))
        self.assertEqual(
            mapping.get(data["robot_dn"]), data["robot_usernames"] + ["cbc"]
        )
